-----------------------------------------------------------------------------
-- (c) Copyright IBM Corp. 2007 All rights reserved.
-- 
-- The following sample of source code ("Sample") is owned by International 
-- Business Machines Corporation or one of its subsidiaries ("IBM") and is 
-- copyrighted and licensed, not sold. You may use, copy, modify, and 
-- distribute the Sample in any form without payment to IBM, for the purpose of 
-- assisting you in the development of your applications.
-- 
-- The Sample code is provided to you on an "AS IS" basis, without warranty of 
-- any kind. IBM HEREBY EXPRESSLY DISCLAIMS ALL WARRANTIES, EITHER EXPRESS OR 
-- IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF 
-- MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. Some jurisdictions do 
-- not allow for the exclusion or limitation of implied warranties, so the above 
-- limitations or exclusions may not apply to you. IBM shall not be liable for 
-- any damages you suffer as a result of using, copying, modifying or 
-- distributing the Sample, even if IBM has been advised of the possibility of 
-- such damages.
-----------------------------------------------------------------------------
--
-- SOURCE FILE NAME: truncate.db2
--
-- SAMPLE: How to implemenet truncate table functionality.
--
-- SQL STATEMENTS USED:
--	   DROP PROCEDURE
--	   DROP TABLE
--         CALL
--         CONNECT
--         CREATE PROCEDURE
--         CREATE TABLE
--         INSERT INTO
--         SELECT
--         TERMINATE
--
-- Note: Use following command to execute the sample:
--         db2 -td@ -vf truncate.db2
--
-- OUTPUT FILE: truncate.out (available in the online documentation)
-----------------------------------------------------------------------------
--
-- For more information about the command line processor (CLP) scripts,
-- see the README file.
--
-- For information on using SQL statements, see the SQL Reference.
--
-- For the latest information on programming, building, and running DB2
-- applications, visit the DB2 application development website:
--     http://www.software.ibm.com/data/db2/udb/ad
-----------------------------------------------------------------------------

-- Connect to 'sample' database
CONNECT TO SAMPLE@

-- 'TRUNCATE' stored procedure removes all rows from the table without using
-- DELETE operation thereby eliminating extensive logging and resulting in
-- better performance.
-- Create a stored Procedure 'truncate' to implement the truncate table
-- functionality. Use IMPORT with REPLACE INTO clause to implement truncate.
-- IMPORT is done using ADMIN_CMD in SQL. Use '/dev/null'(for UNIX) &
-- 'NUL'(for windows) for IMPORT as this file always exists & does not
-- contain anything.

-- Create the stored procedure 'truncate'

CREATE PROCEDURE truncate(IN sch_name VARCHAR(130),IN tab_name VARCHAR(130)) 
  LANGUAGE SQL
  ---------------------------------
  -- SQL Stored Procedure truncate
  ---------------------------------
  BEGIN

    DECLARE stmt VARCHAR(1000);
    DECLARE param VARCHAR(1000);
    DECLARE full_name VARCHAR(1000);
    DECLARE a VARCHAR(130);

    IF sch_name IS NULL 
      THEN
        SET full_name = tab_name;

        -- Check whether the table exists or not
        SELECT tabname INTO a
          FROM SYSCAT.TABLES
          WHERE tabname = UCASE(tab_name);

    ELSE
      SET full_name = sch_name||'.'||tab_name;

      -- Check whether the table exists or not
      SELECT tabname INTO a
        FROM SYSCAT.TABLES
        WHERE tabname = UCASE(tab_name) AND tabschema = UCASE(sch_name);

    END IF;

    IF UCASE(a) = UCASE(tab_name) 
      THEN
        SET param = 'IMPORT FROM /dev/null OF DEL REPLACE INTO '||full_name;

        SET stmt = 'CALL SYSPROC.ADMIN_CMD (?)';
 
        PREPARE s1 FROM stmt;
        EXECUTE s1 USING param;

    ELSE

      -- Table does not exists.

    END IF;

  END @

-- Create and insert some values into the table 'tab1'
CREATE TABLE tab1 ( col1  INTEGER, col2  VARCHAR(130) )@

INSERT INTO tab1 VALUES ( 1, 'some data' ), ( 2, NULL )@

-- Verify the current contents of table tab1
SELECT * FROM tab1@

-- Call the truncate stored procedure for the DB2INST1 schema, and the 
-- table 'tab1'
CALL truncate(CURRENT SCHEMA, 'tab1')@

-- Verify that the table contents have been truncated.
SELECT * FROM tab1@

-- Insert some new values into the tab1 table
INSERT INTO tab1 VALUES ( 2, 'some new data' ), ( 3, NULL )@

SELECT * FROM tab1@

-- Call the truncate procedure with a NULL schema
CALL truncate(NULL, 'tab1')@

-- Verify that the table contents have been truncated.
SELECT * FROM tab1@

-- Drop the table tab1
DROP TABLE tab1@

-- Drop the procedure TRUNCATE
DROP PROCEDURE TRUNCATE@

-- Disconnect from the sample database
CONNECT RESET@

TERMINATE@
